/**************************************************************************\
|
| Version
| =======
| version 1.0, January 2012.
| 
| Copyright
| =========
|  Marc Stevens, 2012. All rights reserved.
| Contact: marc @ marc-stevens .nl
| 
| Disclaimer
| ==========
| This software is provided as is. Use is at the user's risk.
| No guarantee whatsoever is given on how it may function or malfunction.
| Support cannot be expected.
| This software is meant for scientific and educational purposes only.
| It is forbidden to use it for other than scientific or educational purposes.
| In particular, commercial and malicious use is not allowed.
| Further distribution of this software, by whatever means, is not allowed
| without our consent.
| This includes publication of source code or executables in printed form,
| on websites, newsgroups, CD-ROM's, etc.
| Changing the (source) code without our consent is not allowed.
| In all versions of the source code this disclaimer, the copyright
| notice and the version number should be present.
|
\**************************************************************************/

/* 

very fast inlined xorshift random number generators with periods 2^32 - 1, 2^64 - 1, 2^96 - 1 and 2^128 - 1
by G. Marsaglia: http://www.jstatsoft.org/v08/i14/xorshift.pdf 
this implementation only uses time to seed!

*/

#ifndef HASHATTACK_XORSHIFT_RNG_HPP
#define HASHATTACK_XORSHIFT_RNG_HPP

	
	// seed all generators using 32-bit values
	// seed state is deterministically dependent on the given values
	void seed(uint32 s);
	void seed(const uint32* sbuf, unsigned len);
	// add seed to the generators
	// seed state is changed by given values
	void addseed(uint32 s);
	void addseed(const uint32* sbuf, unsigned len);

	// seeds used, these are initialized to random values based on the time
	extern uint32 seedd;
	extern uint32 seed32_1;
	extern uint32 seed32_2;
	extern uint32 seed32_3;
	extern uint32 seed32_4;

	/******** Random generator with perdiod (2^32 - 1)*2^32 **********/
	inline uint32 xrng32()
	{
		seed32_1 ^= seed32_1 << 13;
		seed32_1 ^= seed32_1 >> 17;
		return (seed32_1 ^= seed32_1 << 5) + (seedd += 789456123);
	}

	/******** Random generator with perdiod (2^64 - 1)*2^32 **********/
	inline uint32 xrng64()
	{
		uint32 t = seed32_1 ^ (seed32_1 << 10);
		seed32_1 = seed32_2;
		seed32_2 = (seed32_2^(seed32_2>>10))^(t^(t>>13));
		return seed32_1 + (seedd += 789456123);
	}

	/******** Random generator with perdiod (2^96 - 1)*2^32 **********/
	inline uint32 xrng96()
	{
		uint32 t = seed32_1 ^ (seed32_1 << 10);
		seed32_1 = seed32_2;
		seed32_2 = seed32_3;
		seed32_3 = (seed32_3^(seed32_3>>26))^(t^(t>>5));
		return seed32_1 + (seedd += 789456123);
	}

	/******** Random generator with perdiod (2^128 - 1)*2^32 **********/
	inline uint32 xrng128()
	{
		uint32 t = seed32_1 ^ (seed32_1 << 5);
		seed32_1 = seed32_2;
		seed32_2 = seed32_3;
		seed32_3 = seed32_4;
		seed32_4 = (seed32_4^(seed32_4>>1))^(t^(t>>14));
		return seed32_1 + (seedd += 789456123);
	}

	// call this in a other global constructor using any xrng (without seeding itself)
	// due to the inpredictable order of which global constructors are called
	// otherwise the correct seeding of xrng is not guaranteed
	void hashutil5_rng_hpp_init();

#endif // HASHATTACK_XORSHIFT_RNG_HPP

